@@ -4,48 +4,44 @@ This is a library of tools for creating and manipulating complex networks.
 From [Wikipedia](https://en.wikipedia.org/wiki/Complex_network):
 
 > Complex network is a graph-like structure with non-trivial characteristics.
-> A complex network is a graph (network) with non-trivial topological features—features that do not
-> occur in simple networks such as lattices or random graphs but often occur in networks representing
-> real systems. The study of complex networks is a young and active area of scientific
-> research inspired largely by empirical findings of real-world networks such
-> as computer networks, biological networks, technological networks, brain networks, climate networks
-> and social networks.
+> A complex network is a graph (network) with non-trivial topological features—features that do
+> not occur in simple networks such as lattices or random graphs but often occur in networks
+> representing real systems. The study of complex networks is a young and active area of scientific
+> research inspired largely by empirical findings of real-world networks such as computer networks,
+> biological networks, technological networks, brain networks, climate networks and social networks.
 */
-mod network;
 mod model;
+mod network;
+mod vecmap;
 mod weight;
 
 pub mod bfs;
 pub mod locate;
+pub mod misc;
 pub mod si;
 
+pub use model::Model;
 pub use network::Network;
 pub use weight::Weight;
-pub use model::Model;
 
 pub mod cn {
     //! Library-specific utility module.
     //!
     //! This module serves as a namespace for the various utilities with generic names.
 
+    pub use crate::vecmap::VecMap;
+    pub use crate::vecmap::VecSet;
+    use rustc_hash::FxHasher;
     use std::hash::BuildHasherDefault as Builder;
 
-    /// Re-export of
-    /// [`rustc_hash::FxHasher`](https://docs.rs/rustc-hash/1/rustc_hash/struct.FxHasher.html).
-    pub type FxHasher = rustc_hash::FxHasher;
-
     /// Variant of [`indexmap::IndexMap`] using [`FxHasher`].
     pub type IndexMap<K, V> = indexmap::IndexMap<K, V, Builder<FxHasher>>;
-
-    /// Variant of [`std::collections::HashMap`] using [`FxHasher`].
-    pub type HashMap<K, V> = std::collections::HashMap<K, V, Builder<FxHasher>>;
-
-    /// Variant of [`std::collections::HashSet`] using [`FxHasher`].
-    pub type HashSet<V> = std::collections::HashSet<V, Builder<FxHasher>>;
+    /// Variant of [`indexmap::IndexSet`] using [`FxHasher`].
+    pub type IndexSet<T> = indexmap::IndexSet<T, Builder<FxHasher>>;
 
     /// Shorthand trait for **exact size**, **double-ended** iterator.
-    pub trait Iter<I>: DoubleEndedIterator<Item = I> + ExactSizeIterator<Item = I> {}
-    impl<T: DoubleEndedIterator + ExactSizeIterator + Iterator<Item = I>, I> Iter<I> for T {}
+    pub trait Iter<I>: DoubleEndedIterator<Item = I> {}
+    impl<T: DoubleEndedIterator + Iterator<Item = I>, I> Iter<I> for T {}
 
     /// Error enum used for various recoverable errors
     #[derive(Debug, PartialEq)]
@@ -63,6 +59,8 @@ pub mod cn {
         /// Emitted when an attempt is made to run an algorithm requiring at least 1 target without
         /// targets.
         NoTarget,
+        /// Emitted when the network is required to be whole, but is not.
+        NotWhole,
     }
 
     /// A variant of [`std::result::Result`] which returns [`cn::Err`](crate::cn::Err).
@@ -77,6 +75,7 @@ pub mod cn {
                 Err::BadProbability(p) => format!("Probability cannot be {}", p),
                 Err::NodeExists(i) => format!("Node {} already exists", i),
                 Err::NoTarget => "No search targets were specified".to_string(),
+                Err::NotWhole => "The network must be whole".to_string(),
             };
             write!(f, "{}", message)
         }
