@@ -5,25 +5,75 @@ use rand::distributions::Alphanumeric;
 use rand::prelude::{thread_rng, IteratorRandom, Rng};
 use rand_seeder::Seeder;
 use rand_xoshiro::Xoshiro256Plus;
+use std::hash::{Hash, Hasher};
 use std::sync::{LockResult, Mutex, MutexGuard, TryLockResult};
 
 /// `Node` is a single vertex in the graph and an element of the adjacency list.
 #[derive(Debug, Clone)]
-struct Node(pub cn::IndexMap<usize, f64>);
+struct Node(cn::IndexMap<usize, f64>);
 
-impl Node {
+impl Default for Node {
     /// Creates a new node with no links.
-    fn new() -> Self {
+    fn default() -> Self {
         Node(cn::IndexMap::default())
     }
 }
 
+/// `Link` is an internal representation of an edge in the graph.
+///
+/// The targets are **unordered**, and as such `Link(i, j) == Link(j, i)` and `hash(Link(i, j)) ==
+/// hash(Link(j, i))`.
+#[derive(Debug, Clone)]
+pub struct Link(usize, usize);
+
+impl PartialEq for Link {
+    fn eq(&self, other: &Link) -> bool {
+        (self.0, self.1) == (other.0, other.1) || (self.0, self.1) == (other.1, other.0)
+    }
+}
+
+impl Eq for Link {}
+
+impl Hash for Link {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        // We achieve unordered hash equality by always hashing the LARGER target first.
+        if self.0 >= self.1 {
+            self.0.hash(state);
+            self.1.hash(state);
+        } else {
+            self.1.hash(state);
+            self.0.hash(state);
+        }
+    }
+}
+
+#[test]
+fn test_link_eq_hash() {
+    let mut set = cn::IndexSet::default();
+
+    assert_eq!(Link(1, 1), Link(1, 1));
+    set.insert(Link(1, 1));
+    assert!(set.contains(&Link(1, 1)));
+    assert!(set.remove(&Link(1, 1)));
+
+    let l1 = Link(1, 2);
+    let l2 = Link(2, 1);
+    assert_eq!(l1, l2);
+
+    set.insert(l1);
+    assert!(set.contains(&l2));
+    assert!(set.remove(&l2));
+}
+
 /// `Network` is the main graph data structure. Internally it is represented as an [adjacency
 /// list](https://en.wikipedia.org/wiki/Adjacency_list) of node objects.
 #[derive(Debug)]
 pub struct Network {
     /// The internal adjacency list graph representation.
-    nodes: cn::IndexMap<usize, Node>,
+    nodes: cn::VecMap<Node>,
+
+    /// The internal edge list, useful in some situations.
+    links: cn::IndexSet<Link>,
 
     /// The model used to initialize links between nodes.
     pub(crate) model: Model,
@@ -43,14 +93,16 @@ impl Network {
     /// [`Model::None`] if you want a network with no pre-established connections.
     ///
     /// Note that this is equivalent to calling [`Network::with_seed()`] with a random alphanumeric
-    /// seed of length 16.
+    /// seed of length 64.
     ///
     /// # Examples
     /// Creating a network of chosen model and random link weight:
     /// ```
     /// # use cnetworks::*;
-    /// let net = Network::new(20, Model::ER { p: 0.4, whole: true }, Weight::Uniform);
-    /// assert_eq!(net.size(), 20);
+    /// let net = Network::new(10, Model::ER { p: 0.4, whole: true }, Weight::Uniform);
+    /// assert!(net.is_whole());
+    /// assert_eq!(net.seed().len(), 64);
+    /// assert_eq!(net.size(), 10);
     /// println!("{:?}", net);
     /// ```
     /// Creating a network with no links and establishing them "by hand":
@@ -61,6 +113,7 @@ impl Network {
     /// let mut net = Network::new(10, Model::None, Weight::Constant { c: 0.25 });
     /// net.link(1, 2)?;
     /// net.link(4, 7)?;
+    /// assert!(!net.is_whole());
     /// assert_eq!(net.get_link(1,2)?, Some(0.25));
     /// assert_eq!(net.get_link(4,7)?, Some(0.25));
     /// println!("{:?}", net);
@@ -74,7 +127,7 @@ impl Network {
             weight,
             &thread_rng()
                 .sample_iter(&Alphanumeric)
-                .take(16)
+                .take(64)
                 .map(char::from)
                 .collect::<String>(),
         )
@@ -103,15 +156,16 @@ impl Network {
         }
         // Create the actual network
         let mut net = Network {
-            model: Model::None,
-            nodes: cn::IndexMap::default(),
+            model: model.clone(),
+            nodes: cn::VecMap::with_capacity(size),
+            links: cn::IndexSet::default(),
             weight,
             rng: Mutex::new(Seeder::from(seed).make_rng()),
             seed: seed.to_string(),
         };
         // Fill in the nodes
         for i in 0..size {
-            net.nodes.insert(i, Node::new());
+            net.nodes.insert(i, Node::default());
         }
         // Initialize links according to the desired model
         match model {
@@ -127,7 +181,7 @@ impl Network {
 
     /// Returns the network size, ie. the total number of nodes.
     pub fn size(&self) -> usize {
-        self.nodes().len()
+        self.nodes.len()
     }
 
     /// Returns the [`Model`] of the network.
@@ -153,7 +207,7 @@ impl Network {
         &self.weight
     }
 
-    /// Get the network-local rng thread seed. This is a random alphanumeric string of length 16,
+    /// Get the network-local rng thread seed. This is a random alphanumeric string of length 64,
     /// unless explicitly specified during construcion with [`Network::with_seed()`].
     pub fn seed(&self) -> &str {
         &self.seed
@@ -171,24 +225,88 @@ impl Network {
         self.rng.get_mut()
     }
 
+    /**
+    Scrambles the built-in random number generator with a new random seed. Useful for when a cloned
+    network needs a different RNG source.
+
+    # Examples
+    ```
+    # use cnetworks::*;
+    # use rand::prelude::*;
+    let net = Network::default();
+    let mut net2 = net.clone();
+
+    assert_eq!(net.seed(), net2.seed());
+    assert_eq!(net.rng_lock().unwrap().next_u32(), net2.rng_lock().unwrap().next_u32());
+
+    net2.scramble_rng();
+    assert_ne!(net.seed(), net2.seed());
+    assert_ne!(net.rng_lock().unwrap().next_u32(), net2.rng_lock().unwrap().next_u32());
+    ```
+    */
+    pub fn scramble_rng(&mut self) {
+        let seed = thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(64)
+            .map(char::from)
+            .collect::<String>();
+        self.seed = seed.clone();
+        self.rng = Mutex::new(Seeder::from(seed).make_rng());
+    }
+
     /// Returns iterator with the node indexes.
     ///
+    /// The iteration order is **deterministic**, but not stable, as it is deterministaically
+    /// perturbed when removing links.
+    ///
     /// Keep in mind that it's possible to have a network of nodes with non-sequential and
     /// non-ordered indexes, such as `(0,2,3)`, `(100, 101, 150)` or `(10, 0, 1)` using
     /// [`Network::attach`] and [`Network::`detach`].
     pub fn nodes(&self) -> impl cn::Iter<usize> + '_ {
-        self.nodes.keys().copied()
+        self.nodes.indexes()
     }
 
-    /// Return whether a given nodes exists in the network.
-    pub fn contains(&self, index: usize) -> bool {
-        self.nodes.contains_key(&index)
+    /// Returns iterator of links in the network as (first index, second index) pairs. This is
+    /// intended mostly for explicit iteration over network links, as using [`Network::get_link`]
+    /// or [`Network::links_of`] will be faster than [`Iterator::find`] and others.
+    ///
+    /// The iteration order is **deterministic**, but not stable, as it is deterministaically
+    /// perturbed when removing links.
+    ///
+    /// # Examples
+    /// One common use case is counting edges in the graph.
+    /// ```
+    /// # use cnetworks::*;
+    /// let net = Network::new(100, Model::ER { p: 0.05, whole: true }, Weight::default());
+    /// println!("Network has a total of {} links", net.links().count());
+    /// ```
+    pub fn links(&self) -> impl cn::Iter<(usize, usize)> + '_ {
+        self.links.iter().map(|l| (l.0, l.1))
     }
 
-    /// Returns `true` if the network is whole, ie. if there is only one cluster, or if the network
-    /// is empty.
+    /// Return `true` if a node with specified `index` exists in the network.
+    pub fn exists(&self, index: usize) -> bool {
+        self.nodes.contains(index)
+    }
+
+    /// Returns `true` if the network is whole, ie. if there is only one component or if the
+    /// network is empty.
     pub fn is_whole(&self) -> bool {
-        self.clusters().len() <= 1
+        // Empty network is considered whole
+        if self.size() == 0 {
+            true
+        } else {
+            // We only need to check if every node is reachable from the first one
+            // (and we know there is a first one)
+            self.size()
+                == bfs::reach_many(
+                    self,
+                    self.nodes().next().unwrap(),
+                    &self.nodes().collect::<Vec<_>>(),
+                )
+                .unwrap_or_default()
+                .len()
+        }
     }
 
     /// Returns the degree of a given `node`, ie. the number of it's closest neighbors.
@@ -202,7 +320,7 @@ impl Network {
     ///
     /// Returns [`cn::Err::NoSuchNode`] if specified node does not exist.
     pub fn links_of(&self, node: usize) -> cn::Result<&cn::IndexMap<usize, f64>> {
-        match self.nodes.get(&node) {
+        match self.nodes.get(node) {
             Some(node) => Ok(&node.0),
             None => Err(cn::Err::NoSuchNode(node)),
         }
@@ -213,7 +331,7 @@ impl Network {
     /// Returns [`cn::Err::LinkToSelf`] if `i` and `j` are the same, [`cn::Err::NoSuchNode`] if
     /// either `i` or `j` does not exist.
     pub fn get_link(&self, i: usize, j: usize) -> cn::Result<Option<f64>> {
-        if !self.contains(j) {
+        if !self.exists(j) {
             return Err(cn::Err::NoSuchNode(j));
         }
         let link = self.links_of(i)?.get(&j).cloned();
@@ -230,11 +348,6 @@ impl Network {
             .count()
     }
 
-    /// Returns the total number of edges in the network.
-    pub fn edges(&self) -> usize {
-        self.deg_total() / 2
-    }
-
     /// Returns the arithmetical average of node degrees in the network.
     pub fn deg_avg(&self) -> f64 {
         self.deg_total() as f64 / self.size() as f64
@@ -245,12 +358,16 @@ impl Network {
         self.nodes().map(|node| self.deg_of(node).unwrap()).sum()
     }
 
-    /// Returns the histogram of node degrees as a [`cn::HashMap`] of (degree, occurrences) pairs.
-    pub fn deg_distr(&self) -> cn::HashMap<usize, usize> {
-        let mut bins: cn::HashMap<usize, usize> = cn::HashMap::default();
+    /// Returns the histogram of node degrees as a [`cn::VecMap`] of (degree, occurrences) pairs.
+    pub fn deg_distr(&self) -> cn::VecMap<usize> {
+        let mut bins: cn::VecMap<usize> = cn::VecMap::default();
         for node in self.nodes() {
             let deg = self.deg_of(node).unwrap();
-            *bins.entry(deg).or_insert(0) += 1;
+            if let Some(&count) = bins.get(deg) {
+                bins.insert(deg, count + 1);
+            } else {
+                bins.insert(deg, 1);
+            }
         }
         bins
     }
@@ -276,23 +393,22 @@ impl Network {
     /// Returns [`cn::Err::NoSuchNode`] if `i` and `j` do not exist, [`cn::Err::LinkToSelf`] if they
     /// are the same, [`cn::Err::BadWeight`] the `weight` is not in the range (0, 1].
     pub fn link_exact(&mut self, i: usize, j: usize, weight: f64) -> cn::Result<Option<f64>> {
-        let old_weight = self.get_link(i, j)?;
+        // All kinds of input error
         if weight <= 0. || weight > 1. {
             return Err(cn::Err::BadWeight(weight));
         }
         if i == j {
             return Err(cn::Err::LinkToSelf(i));
         }
-        if !self.contains(i) {
-            return Err(cn::Err::NoSuchNode(i));
+        match (self.nodes.contains(i), self.nodes.contains(j)) {
+            (true, true) => {
+                self.links.insert(Link(i, j));
+                self.nodes[j].0.insert(i, weight);
+                Ok(self.nodes[i].0.insert(j, weight))
+            }
+            (true, false) => Err(cn::Err::NoSuchNode(j)),
+            (false, _) => Err(cn::Err::NoSuchNode(i)),
         }
-        if !self.contains(j) {
-            return Err(cn::Err::NoSuchNode(j));
-        }
-        self.nodes[&i].0.insert(j, weight);
-        self.nodes[&j].0.insert(i, weight);
-
-        Ok(old_weight)
     }
 
     /// Unsafely removes a link between nodes `i` and `j`. Does not perform a check whether the
@@ -302,13 +418,17 @@ impl Network {
     /// Returns [`cn::Err::NoSuchNode`] if `i` and `j` do not exist and [`cn::Err::LinkToSelf`] if they
     /// are the same.
     pub fn unlink(&mut self, i: usize, j: usize) -> cn::Result<Option<f64>> {
-        match self.get_link(i, j)? {
-            Some(weight) => {
-                self.nodes[&i].0.remove(&j);
-                self.nodes[&j].0.remove(&i);
-                Ok(Some(weight))
+        if i == j {
+            return Err(cn::Err::LinkToSelf(i));
+        }
+        match (self.nodes.contains(i), self.nodes.contains(j)) {
+            (true, true) => {
+                self.links.remove(&Link(i, j));
+                self.nodes[j].0.remove(&i);
+                Ok(self.nodes[i].0.remove(&j))
             }
-            None => Ok(None),
+            (true, false) => Err(cn::Err::NoSuchNode(j)),
+            (false, _) => Err(cn::Err::NoSuchNode(i)),
         }
     }
 
@@ -324,13 +444,11 @@ impl Network {
             Some(weight) => weight,
             None => return Ok(None),
         };
-        match bfs::distance(self, i, j)? {
-            Some(..) => Ok(Some(w)),
-            None => {
-                // There is no longer a path from i to j - revert the unlink
-                self.link_exact(i, j, w)?;
-                Ok(None)
-            }
+        if bfs::reach(self, i, j)? {
+            Ok(Some(w))
+        } else {
+            self.link_exact(i, j, w)?;
+            Ok(None)
         }
     }
 
@@ -340,12 +458,16 @@ impl Network {
     ///
     /// Returns [`cn::Err::NoSuchNode`] if `node` does not exist.
     pub fn disconnect(&mut self, node: usize) -> cn::Result<cn::IndexMap<usize, f64>> {
-        let old_links = self.links_of(node)?.clone();
-        self.nodes[&node].0.clear();
-        for &idx in old_links.keys() {
-            self.nodes[&idx].0.remove(&node);
+        if let Some(node_obj) = self.nodes.get_mut(node) {
+            let old_links = std::mem::take(&mut node_obj.0);
+            for &other in old_links.keys() {
+                self.nodes[other].0.remove(&node);
+                self.links.remove(&Link(other, node));
+            }
+            Ok(old_links)
+        } else {
+            Err(cn::Err::NoSuchNode(node))
         }
-        Ok(old_links)
     }
 
     /// Safely disconnects a `node` from the network by removing all of its links. Performs a
@@ -358,21 +480,19 @@ impl Network {
         // Blindly disconnect the requested node
         let links = self.disconnect(node)?;
         // Get a vector of used-to-be-connected nodes
-        let used_to: Vec<&usize> = links.keys().collect();
-        // i and j are indexes in the keys vector and the values of keys[i] and keys[j] are
-        // the used-to-be-connected nodes in self.nodes
-        for (i, &node_i) in used_to.iter().enumerate() {
-            // For every node that used to connect to the removed...
-            for &node_j in &used_to[i..] {
-                // ... for all nodes after it in the used_to vector...
-                // ... check if path between them still exists
-                if bfs::distance(self, *node_i, *node_j)?.is_none() {
-                    // There is no longer a path from node_i to node_j - revert the disconnect
-                    for (key, value) in links {
-                        self.link_exact(key, node, value)?;
-                    }
-                    return Ok(None);
+        let link_vec: Vec<_> = links.keys().copied().collect();
+        // Check if it's possible to reach all the others from first
+        if let Some((first, others)) = link_vec.split_first() {
+            if bfs::reach_many(self, *first, others)
+                .unwrap_or_default()
+                .len()
+                != others.len()
+            {
+                // Not reachable - reestablish links and return None
+                for (key, value) in links {
+                    self.link_exact(key, node, value)?;
                 }
+                return Ok(None);
             }
         }
         Ok(Some(links))
@@ -383,9 +503,10 @@ impl Network {
     /// Sets the model to [`Model::None`] such that the initial linking process can be conducted
     /// again, eg. with [`Model::init_er`], [`Model::init_ba`] or manually.
     pub fn disconnect_all(&mut self) {
-        for node in &mut self.nodes.values_mut() {
+        for node in (&mut self.nodes.as_vec_mut().iter_mut()).flatten() {
             node.0.clear();
         }
+        self.links.clear();
         self.model = Model::None;
     }
 
@@ -402,10 +523,10 @@ impl Network {
     /// assert!(net.attach(2).is_err());
     /// ```
     pub fn attach(&mut self, node: usize) -> cn::Result<()> {
-        if self.contains(node) {
+        if self.exists(node) {
             return Err(cn::Err::NodeExists(node));
         } else {
-            self.nodes.insert(node, Node::new());
+            self.nodes.insert(node, Node::default());
         }
         Ok(())
     }
@@ -414,8 +535,8 @@ impl Network {
     /// instead of raising an error.
     pub fn attach_skip(&mut self, indexes: &[usize]) {
         for &i in indexes {
-            if !self.contains(i) {
-                self.nodes.insert(i, Node::new());
+            if !self.exists(i) {
+                self.nodes.insert(i, Node::default());
             }
         }
     }
@@ -432,77 +553,108 @@ impl Network {
     /// ```
     pub fn detach(&mut self, node: usize) -> cn::Result<()> {
         self.disconnect(node)?;
-        self.nodes.remove(&node);
+        self.nodes.remove(node);
         Ok(())
     }
 
-    /// Returns vector of [clusters](https://en.wikipedia.org/wiki/Component_(graph_theory))
+    /// Returns vector of [components](https://en.wikipedia.org/wiki/Component_(graph_theory))
     /// present in the network, ordered largest to smallest.
     ///
     /// # Examples
     /// ```
     /// # use cnetworks::*;
     /// let net = Network::new(100, Model::ER { p: 0.05, whole: false }, Weight::default() );
-    /// let clusters = net.clusters();
-    /// println!("Largest cluster: {:?}", clusters.first().expect("No clusters"));
-    /// println!("Smallest cluster: {:?}", clusters.last().expect("No clusters"));
+    /// let components = net.components();
+    /// println!("Largest component: {:?}", components.first().expect("No components"));
+    /// println!("Smallest component: {:?}", components.last().expect("No components"));
     /// ```
-    pub fn clusters(&self) -> Vec<cn::HashSet<usize>> {
-        let mut clusters: Vec<cn::HashSet<usize>> = Vec::new();
+    pub fn components(&self) -> Vec<cn::VecSet> {
+        let mut components: Vec<cn::VecSet> = Vec::new();
         if self.size() == 0 {
-            return clusters;
+            return components;
         }
         // Start from a random node
         let root = self.nodes().choose(&mut *self.rng_lock().unwrap()).unwrap();
         let (expl, mut unexpl) = bfs::explore(self, root).unwrap();
-        // Add explored to the vector of clusters
-        clusters.push(expl);
+        // Add explored to the vector of components
+        components.push(expl);
         loop {
             match unexpl.iter().choose(&mut *self.rng_lock().unwrap()) {
-                Some(&new_root) => {
-                    // Establish its cluster
+                Some(new_root) => {
+                    // Establish its component
                     let (cls, _) = bfs::explore(self, new_root).unwrap();
-                    // Mark everything in the cluster as explored
-                    for c in &cls {
+                    // Mark everything in the component as explored
+                    for c in cls.iter() {
                         unexpl.remove(c);
                     }
-                    clusters.push(cls);
+                    components.push(cls);
                 }
                 // The are no unexplored nodes
                 None => {
-                    // Order cluster largest -> smallest
-                    clusters.sort_by_key(|set| -(set.len() as i64));
-                    return clusters;
+                    // Order components largest -> smallest
+                    components.sort_by_key(|set| -(set.len() as i64));
+                    return components;
                 }
             }
         }
     }
 
-    /// Stitches the network together if it is composed of more than one cluster. Connects random
-    /// elements from smaller clusters to the largest one, extending it until it covers the entire
+    /// Transform the network into the largest contained component by removing all nodes **not** in
+    /// the largest component. Leaves `self` unchanged if there is no largest component.
+    ///
+    /// # Examples
+    /// ```
+    /// # use cnetworks::*;
+    /// # use std::error::Error;
+    /// # fn main() -> Result<(), Box<dyn Error>> {
+    /// let mut net = Network::new(10, Model::None, Weight::default() );
+    /// net.link(0, 1)?;
+    /// net.link(0, 2)?;
+    /// net.link(0, 3)?;
+    /// assert_eq!(net.clone().into_largest_component().links().count(), 3);
+    /// assert_eq!(net.clone().into_largest_component().size(), 4);
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn into_largest_component(mut self) -> Self {
+        if let Some(largest) = self.components().first() {
+            let to_remove: Vec<usize> = self.nodes().filter(|n| !largest.contains(*n)).collect();
+            for not_included in to_remove {
+                self.nodes.remove(not_included);
+            }
+        }
+        self
+    }
+
+    /// Stitches the network together if it is composed of more than one component. Connects random
+    /// elements from smaller components to the largest one, extending it until it covers the entire
     /// network.
     ///
     /// # Examples
     /// ```
     /// # use cnetworks::*;
     /// let mut net = Network::new(100, Model::ER { p: 0.0001, whole: false }, Weight::default() );
-    /// assert_ne!(net.clusters().len(), 1 );
+    /// assert!(!net.is_whole());
+    /// assert_ne!(net.components().len(), 1 );
     /// net.stitch_together();
-    /// assert_eq!(net.clusters().len(), 1 );
+    /// assert!(net.is_whole());
+    /// assert_eq!(net.components().len(), 1 );
     /// ```
     pub fn stitch_together(&mut self) {
-        let mut clusters = self.clusters();
-        let (largest, others) = match clusters.split_first_mut() {
+        let mut components = self.components();
+        let (largest, others) = match components.split_first_mut() {
             Some(clst) => clst,
             None => return,
         };
-        for cluster in others {
-            // Connect random node from current cluster to random node from the largest cluster
-            let current = cluster.iter().choose(&mut *self.rng().unwrap()).unwrap();
+        for component in others {
+            // Connect random node from current component to random node from the largest component
+            let current = component.iter().choose(&mut *self.rng().unwrap()).unwrap();
             let other = largest.iter().choose(&mut *self.rng().unwrap()).unwrap();
-            self.link(*current, *other).unwrap();
-            // Extend the largest cluster with the just-connected nodes
-            largest.extend(cluster.iter());
+            self.link(current, other).unwrap();
+            // Extend the largest component with the just-connected nodes
+            for node in component.iter() {
+                largest.insert(node);
+            }
         }
     }
 }
@@ -524,11 +676,13 @@ impl Default for Network {
     /// # use std::error::Error;
     /// # fn main() -> Result<(), Box<dyn Error>> {
     /// let mut net = Network::default();
+    /// assert!(net.is_whole()); // empty network is considered whole
     /// for i in [1,2] {
-    /// net.attach(i)?;
+    ///     net.attach(i)?;
     /// }
     /// net.link(1,2)?;
     /// assert_eq!(net.get_link(1,2), Ok(Some(1.0)));
+    /// assert!(net.is_whole());
     /// # Ok(())
     /// # }
     /// ```
@@ -541,6 +695,7 @@ impl Clone for Network {
     fn clone(&self) -> Self {
         Network {
             nodes: self.nodes.clone(),
+            links: self.links.clone(),
             model: self.model.clone(),
             weight: self.weight.clone(),
             rng: Mutex::new(self.rng.try_lock().unwrap().clone()),
