@@ -11,7 +11,7 @@ use rand::prelude::{IteratorRandom, Rng};
 pub struct Observers {
     true_source: usize,
     true_time: usize,
-    observers: cn::IndexMap<usize, Option<usize>>,
+    observers: cn::IndexMap<usize, usize>,
 }
 
 impl Observers {
@@ -25,8 +25,9 @@ impl Observers {
         self.true_time
     }
 
-    /// Returns immutable reference to the wrapped [`cn::IndexMap`].
-    pub fn as_map(&self) -> &cn::IndexMap<usize, Option<usize>> {
+    /// Returns immutable reference to the wrapped [`cn::IndexMap`] of `(node index, infection
+    /// time)` pairs.
+    pub fn as_map(&self) -> &cn::IndexMap<usize, usize> {
         &self.observers
     }
 
@@ -74,6 +75,13 @@ impl Observers {
     pub fn keep_portion(&mut self, portion: f64, rng: &mut impl Rng) -> &mut Self {
         self.keep_random((portion * self.as_map().len() as f64).round() as usize, rng)
     }
+
+    /// Keep only observers found in `net`, discarding all of the others.
+    ///
+    /// Returns mutable reference to `self` for easier method chaining.
+    pub fn adjust_to(&mut self, net: &Network) -> &mut Self {
+        self.keep(&net.nodes().collect::<Vec<usize>>())
+    }
 }
 
 /// Initialize the SI information spread and perform a number of synchronous `steps`, starting from
@@ -82,20 +90,19 @@ impl Observers {
 /// Returns [`cn::Err::NoSuchNode`] if `source` does not exist.
 pub fn spread(net: &Network, source: usize, steps: usize) -> cn::Result<Observers> {
     let mut infected = cn::IndexMap::default();
-    infected.insert(source, Some(0));
+    infected.insert(source, 0);
     for step in 1..steps + 1 {
         let inf: Vec<usize> = infected.keys().copied().collect();
         for i in inf {
             for (neighbor, weight) in net.links_of(i)? {
                 if let Entry::Vacant(entry) = infected.entry(*neighbor) {
                     if net.rng_lock().unwrap().gen::<f64>() < *weight {
-                        entry.insert(Some(step));
+                        entry.insert(step);
                     }
                 }
             }
         }
     }
-    infected.remove(&source);
     Ok(Observers {
         true_source: source,
         true_time: 0,
