@@ -26,12 +26,12 @@ fn setup_net(c: f64, seed: &str) -> Network {
 #[test]
 fn test_pearson() {
     for c in [0.5, 0.75, 1.0] {
-        let net = setup_net(c, "O2jsYh2AQ6BrLWJS");
-        let keep = [1, 2, 3];
+        let net = setup_net(c, "test seed");
         let mut observers = si::spread(&net, 5, 10).unwrap();
-        observers.keep(&keep);
+        observers.keep(&[0, 1, 2, 3]);
         let result = locate::pearson(&net, &observers).unwrap();
         let (most_likely, _score) = result.top().unwrap();
+        dbg!(result);
         assert!(most_likely == 5);
     }
 }
@@ -39,10 +39,9 @@ fn test_pearson() {
 #[test]
 fn test_pinto() {
     for c in [0.5, 0.75, 1.0] {
-        let net = setup_net(c, "O2jsYh2AQ6BrLWJS");
-        let keep = [1, 2, 3];
+        let net = setup_net(c, "test seed");
         let mut observers = si::spread(&net, 5, 10).unwrap();
-        observers.keep(&keep);
+        observers.keep(&[0, 1, 2, 3]);
         let result = locate::lptv(&net, &observers).unwrap();
         let (most_likely, _score) = result.top().unwrap();
         assert!(most_likely == 5);
@@ -67,7 +66,7 @@ fn test_pearson_stats() {
             observers.keep_portion(0.1, &mut *net.rng_lock().unwrap()),
         )
         .unwrap();
-        acc += result.accuracy();
+        acc += result.precision();
     }
     acc /= 5000.0;
     assert!(acc > 0.95);
@@ -91,8 +90,9 @@ fn test_pinto_stats() {
             observers.keep_portion(0.1, &mut *net.rng_lock().unwrap()),
         )
         .unwrap();
-        acc += result.accuracy();
+        acc += result.precision();
     }
     acc /= 5000.0;
+    dbg!(acc);
     assert!(acc > 0.95);
 }
