@@ -5,10 +5,10 @@ use cnetworks::*;
 fn test_get() {
     let net = Network::new(3, Model::None, Weight::default());
     for i in 0..3 {
-        assert!(net.contains(i));
+        assert!(net.exists(i));
     }
     for i in 3..100 {
-        assert!(!net.contains(i));
+        assert!(!net.exists(i));
     }
 }
 
@@ -34,7 +34,7 @@ fn test_degs() {
 
     assert_eq!(net.deg_of(2).unwrap(), 1);
     assert_eq!(net.deg_total(), 4);
-    assert_eq!(net.edges(), 2);
+    assert_eq!(net.links().count(), 2);
 
     assert_eq!(net.deg_distr(), vec![(1, 2), (2, 1)].into_iter().collect());
 }
@@ -44,5 +44,6 @@ fn test_model() {
     let mut net = Network::new(3, Model::None, Weight::default());
     let name = "custom model name".to_string();
     net.set_model(&name);
-    assert_eq!(net.model(), &Model::Custom(name));
+    assert_eq!(net.model(), &Model::Custom(name.clone()));
+    assert_eq!(net.model().as_str(), name);
 }
