@@ -11,17 +11,15 @@ fn test_spread_random() {
     let mut infected = observers.as_map().keys().copied().collect::<Vec<usize>>();
     infected.sort_unstable();
     match observers.true_source() {
-        0 => assert_eq!(infected, vec![1]),
-        1 => assert_eq!(infected, vec![0, 2]),
-        2 => assert_eq!(infected, vec![1, 3]),
-        3 => assert_eq!(infected, vec![2]),
+        0 => assert_eq!(infected, vec![0, 1]),
+        1 => assert_eq!(infected, vec![0, 1, 2]),
+        2 => assert_eq!(infected, vec![1, 2, 3]),
+        3 => assert_eq!(infected, vec![2, 3]),
         _ => panic!("Root should not be {}", observers.true_source()),
     }
     let observers = si::spread_random(&net, 3);
     assert!(vec![0, 1, 2, 3].contains(&observers.true_source()));
     let mut infected = observers.as_map().keys().copied().collect::<Vec<usize>>();
     infected.sort_unstable();
-    let mut cmp = vec![0, 1, 2, 3];
-    cmp.retain(|x| *x != observers.true_source());
-    assert_eq!(infected, cmp)
+    assert_eq!(infected, vec![0, 1, 2, 3])
 }
